describe CrvApiClient::Animals do
  let(:endpoint) { "https://localhost/animals" }
  let(:default_content_type) { 'application/json' }

  let(:animal) {::CrvApiClient::Api::Animals::AnimalId.new("b","c")}
  let(:config_customer) { {participant_code:"10106605",participant_code_type:"PAR", country_code:"NLD"} }
  let(:config_provider) { {participant_code:"2323",participant_code_type:"test", country_code:"asd"} }
  let(:crv_customer) { ::CrvApiClient::Api::Animals::ParticipantId.new("10106605","PAR","NLD")}
  let(:crv_provider) { ::CrvApiClient::Api::Animals::ParticipantId.new("2323","test","asd")}
  let(:username) {"Username"}
  let(:password) {"password"}

  let(:client) { CrvApiClient::Animals.new }
  let(:configured_client) { ::CrvApiClient::Animals.new(endpoint, username: username, password: password, customer: config_customer, provider: config_provider ) }
  context 'initialization' do

    it "should initalize the client with given parameters" do
      expect(configured_client.endpoint_url).to eq(endpoint)
      expect(configured_client.username).to eq(username)
      expect(configured_client.password).to eql(password)
      expect(configured_client.customer).to be_a(::CrvApiClient::Api::Animals::ParticipantId)
      expect(configured_client.provider).to be_a(::CrvApiClient::Api::Animals::ParticipantId)

    end

    it "should initialize with the configured attributes as default" do
      CrvApiClient.configure do |conf|
        conf.endpoint_animals = endpoint
        conf.username = username
        conf.password = password
        conf.customer = config_customer
      end

      expect(client.endpoint_url).to eq(endpoint)
      expect(client.username).to eq(username)
      expect(client.password).to eq(password)
      expect(client.customer).to be_a(::CrvApiClient::Api::Animals::ParticipantId)
    end
  end

  context 'list_animals' do
    it 'should list animals' do
      crv_id = 20
      country_code  = "nl"
      process_code = "2838"
      context_message = CrvApiClient::Api::Animals::ContextMessage.new
      keeper = ::CrvApiClient::Api::Animals::ParticipantId.new(crv_id,"",country_code)
      context_message_detail = ::CrvApiClient::Api::Animals::ContextMessageDetail.new("a","b")
      service_message = ::CrvApiClient::Api::Animals::ServiceMessage
      response = ::CrvApiClient::Api::Animals::ListHerdPedigreeResponse.new(nil,service_message)


      expect(configured_client).to receive(:keeper_and_context_message_detail).with(crv_id, country_code).and_return([keeper,context_message_detail])
      expect(configured_client).to receive(:context_message).with(keeper, username: username, password: password, customer: instance_of(CrvApiClient::Api::Animals::ParticipantId), provider: instance_of(CrvApiClient::Api::Animals::ParticipantId), animal: nil, context_message_detail: context_message_detail,  process_code: process_code, organisation: nil).and_return(context_message)
      driver = configured_client.send(:driver)
      expect(driver).to receive(:listHerdPedigree).with(satisfy { |message|
        expect(message).to be_a(CrvApiClient::Api::Animals::ListHerdPedigreeRequest)
        expect(message.requestMessage).to be_a(CrvApiClient::Api::Animals::ListHerdPedigreeRequestType)
        expect(message.requestMessage.keeper).to be(keeper)
        expect(message.contextMessage).to be(context_message)
      }).and_return(response)
      expect(configured_client).to receive(:perform_service_message_validation).with(service_message)

      response = configured_client.list_animals(crv_id,country_code,process_code: process_code)

    end

    it 'should not handle message validations when option is given.' do
      crv_id = 20
      country_code  = "nl"
      process_code = "2838"
      context_message = CrvApiClient::Api::Animals::ContextMessage.new
      keeper = ::CrvApiClient::Api::Animals::ParticipantId.new(crv_id,"",country_code)
      context_message_detail = ::CrvApiClient::Api::Animals::ContextMessageDetail.new("a","b")
      service_message = ::CrvApiClient::Api::Animals::ServiceMessage
      response = ::CrvApiClient::Api::Animals::ListHerdPedigreeResponse.new(nil,service_message)


      expect(configured_client).to receive(:keeper_and_context_message_detail).with(crv_id, country_code).and_return([keeper,context_message_detail])
      expect(configured_client).to receive(:context_message).with(keeper, username: username, password: password, customer: instance_of(CrvApiClient::Api::Animals::ParticipantId), provider: instance_of(CrvApiClient::Api::Animals::ParticipantId), animal: nil, context_message_detail: context_message_detail,  process_code: process_code, organisation: nil).and_return(context_message)
      driver = configured_client.send(:driver)
      expect(driver).to receive(:listHerdPedigree).with(satisfy { |message|
        expect(message).to be_a(CrvApiClient::Api::Animals::ListHerdPedigreeRequest)
        expect(message.requestMessage).to be_a(CrvApiClient::Api::Animals::ListHerdPedigreeRequestType)
        expect(message.requestMessage.keeper).to be(keeper)
        expect(message.contextMessage).to be(context_message)
      }).and_return(response)
      expect(configured_client).to receive(:perform_service_message_validation).with(service_message).never

      response = configured_client.list_animals(crv_id,country_code,process_code: process_code, options: {validate: false})

    end
  end
  
end