require 'spec_helper'

describe ::CrvApiClient::Helpers::Attributes  do

  let(:tested_class) { Class.new {
    extend ::CrvApiClient::Helpers::Attributes
  }.new }
  context "attr_reader_with_type" do
    it "should create a reader that is public" do
      tested_class.class.attr_reader_with_type :foo , String
      expect(tested_class).to respond_to(:foo).with(0).argument
    end

    it "should create a writer that is private" do
      tested_class.class.attr_reader_with_type :foo , String
      expect(tested_class.private_methods).to include(:foo=)
    end

  end

  context "attr_accessor_with_type" do
    it "should create a reader that is public" do
      tested_class.class.attr_accessor_with_type :foo , String
      expect(tested_class).to respond_to(:foo).with(0).argument
    end

    it "should create a writer that is public" do
      tested_class.class.attr_accessor_with_type :foo , String
      expect(tested_class).to respond_to(:foo=).with(1).argument
    end

  end

  context "general assignment" do
    it "should throw an error when type does not match the assigned type" do
      tested_class.class.attr_accessor_with_type :foo , String
      expect{tested_class.foo = 1}.to raise_error(ArgumentError, /Invalid Type/)
    end

    it "should accept a matching type" do
      tested_class.class.attr_accessor_with_type :foo , String
      expect{tested_class.foo = "some string"}.not_to raise_error
    end

    it "should return the given argument" do
      argument = "Some string"
      tested_class.class.attr_accessor_with_type :foo , String
      expect{tested_class.foo = argument}.not_to raise_error
      expect(tested_class.foo).to be(argument)
    end
  end
end