require 'spec_helper'

describe ::CrvApiClient::Helpers::Endpoint do
  CrvApiClient::Helpers.const_set("SomeClass", Class.new() { include ::CrvApiClient::Helpers::Endpoint } )
  Object.const_set("NoModuleClass", Class.new() { include ::CrvApiClient::Helpers::Endpoint } )
  let(:tested_class) { CrvApiClient::Helpers::SomeClass.new}
  let(:animal_class) { CrvApiClient::Animals.new}
  let(:reproduction_class) { CrvApiClient::Reproduction.new}

  shared_examples 'endpoint resolver' do |klass_under_test,expected_klass, method|
    it "and should create a #{expected_klass} from a #{klass_under_test}"  do
      klass_instance = klass_under_test.new
      klass_instance.extend(::CrvApiClient::Helpers::Endpoint)
      expect(Object).to receive(:const_get).with(expected_klass.name).and_return(expected_klass.new)
      klass = klass_instance.send(method)
      expect(klass).to be_a(expected_klass)
    end
  end

  context 'class_api_endpoint' do
    it "should add the module api before the class name" do
      result = tested_class.send(:class_api_endpoint)
      expect(result).to eq("CrvApiClient::Helpers::Api::SomeClass::")
    end

    it "should return the class when namespace cant be found" do
      result = NoModuleClass.new.send(:class_api_endpoint)
      expect(result).to eq("NoModuleClass")
    end
  end


  context 'endpoint_participant_id' do
      it_behaves_like 'endpoint resolver' , CrvApiClient::Animals, CrvApiClient::Api::Animals::ParticipantId, :endpoint_participant_id
      it_behaves_like 'endpoint resolver' , CrvApiClient::Reproduction, CrvApiClient::Api::Reproduction::ParticipantId, :endpoint_participant_id
  end

  context 'endpoint_context_message_detail' do
    it_behaves_like 'endpoint resolver' , CrvApiClient::Animals, CrvApiClient::Api::Animals::ContextMessageDetail, :endpoint_context_message_detail
    it_behaves_like 'endpoint resolver' , CrvApiClient::Reproduction, CrvApiClient::Api::Reproduction::ContextMessageDetail, :endpoint_context_message_detail
  end

  context 'endpoint_context_message' do
    it_behaves_like 'endpoint resolver' , CrvApiClient::Animals, CrvApiClient::Api::Animals::ContextMessage, :endpoint_context_message
    it_behaves_like 'endpoint resolver' , CrvApiClient::Reproduction, CrvApiClient::Api::Reproduction::ContextMessage, :endpoint_context_message
  end

  context 'endpoint_animal_id' do
    it_behaves_like 'endpoint resolver' , CrvApiClient::Animals, CrvApiClient::Api::Animals::AnimalId, :endpoint_animal_id
    it_behaves_like 'endpoint resolver' , CrvApiClient::Reproduction, CrvApiClient::Api::Reproduction::AnimalId, :endpoint_animal_id
  end
end


