describe CrvApiClient::Reproduction do
  let(:endpoint) { "https://localhost/reproduction" }
  let(:default_content_type) { 'application/json' }

  let(:animal) {::CrvApiClient::Api::Reproduction::AnimalId.new("b","c")}
  let(:config_customer) { {participant_code:"10106605",participant_code_type:"PAR", country_code:"NLD"} }
  let(:config_provider) { {participant_code:"2323",participant_code_type:"test", country_code:"asd"} }
  let(:crv_customer) { ::CrvApiClient::Api::Reproduction::ParticipantId.new("10106605","PAR","NLD")}
  let(:crv_provider) { ::CrvApiClient::Api::Reproduction::ParticipantId.new("2323","test","asd")}
  let(:username) {"Username"}
  let(:password) {"password"}

  let(:client) { CrvApiClient::Reproduction.new }
  let(:configured_client) { ::CrvApiClient::Reproduction.new(endpoint, username: username, password: password, customer: config_customer, provider: config_provider ) }
  context 'initialization' do

    it "should initalize the client with given parameters" do
      expect(configured_client.endpoint_url).to eq(endpoint)
      expect(configured_client.username).to eq(username)
      expect(configured_client.password).to eql(password)
      expect(configured_client.customer).to be_a(::CrvApiClient::Api::Reproduction::ParticipantId)
      expect(configured_client.provider).to be_a(::CrvApiClient::Api::Reproduction::ParticipantId)

    end

    it "should initialize with the configured attributes as default" do
      CrvApiClient.configure do |conf|
        conf.endpoint_reproduction = endpoint
        conf.username = username
        conf.password = password
        conf.customer = config_customer
      end

      expect(client.endpoint_url).to eq(endpoint)
      expect(client.username).to eq(username)
      expect(client.password).to eq(password)
      expect(client.customer).to be_a(::CrvApiClient::Api::Reproduction::ParticipantId)
    end
  end

  context 'list_reproduction_last_parity' do
    it 'should list animals' do
      crv_id = 20
      country_code  = "nl"
      process_code = "2838"
      context_message = CrvApiClient::Api::Reproduction::ContextMessage.new
      keeper = ::CrvApiClient::Api::Reproduction::ParticipantId.new(crv_id,"",country_code)
      context_message_detail = ::CrvApiClient::Api::Reproduction::ContextMessageDetail.new("a","b")
      animal_request = CrvApiClient::Api::Reproduction::ParticipantAnimalRequest.new
      animal_request_list = CrvApiClient::Api::Reproduction::ListParticipantAnimalRequest.new
      requestType = CrvApiClient::Api::Reproduction::ListReproductionLastParityRequestType.new
      service_message = ::CrvApiClient::Api::Reproduction::ServiceMessage
      response = ::CrvApiClient::Api::Reproduction::ListReproductionLastParityResponse.new(nil,service_message)
      driver = configured_client.send(:driver)

      expect(configured_client).to receive(:keeper_and_context_message_detail).with(crv_id, country_code).and_return([keeper,context_message_detail]).once
      expect(configured_client).to receive(:context_message).with(keeper, username: username, password: password, customer: instance_of(CrvApiClient::Api::Reproduction::ParticipantId), provider: instance_of(CrvApiClient::Api::Reproduction::ParticipantId), animal: nil, context_message_detail: context_message_detail,  process_code: process_code, organisation: nil).and_return(context_message).once
      expect(configured_client).to receive(:participant_animal_request).with("TRP","PED").and_return(animal_request).once
      expect(configured_client).to receive(:list_participant_animal_request).with("SHORT",[animal_request]).and_return(animal_request_list).once
      expect(configured_client).to receive(:list_reproduction_last_parity_request_type).with(keeper,animal_request_list).and_return(requestType).once
      expect(driver).to receive(:listReproductionLastParity).with(satisfy { |message|
        expect(message).to be_a(CrvApiClient::Api::Reproduction::ListReproductionLastParityRequest)
        expect(message.requestMessage).to be_a(CrvApiClient::Api::Reproduction::ListReproductionLastParityRequestType)
        expect(message.requestMessage).to be(requestType)
        expect(message.contextMessage).to be(context_message)
      }).and_return(response).once
      expect(configured_client).to receive(:perform_service_message_validation).with(service_message).once

      configured_client.list_reproduction_last_parity(crv_id,country_code,process_code: process_code)

    end

    it 'should not handle message validations when option is given.' do
      crv_id = 20
      country_code  = "nl"
      process_code = "2838"
      context_message = CrvApiClient::Api::Reproduction::ContextMessage.new
      keeper = ::CrvApiClient::Api::Reproduction::ParticipantId.new(crv_id,"",country_code)
      context_message_detail = ::CrvApiClient::Api::Reproduction::ContextMessageDetail.new("a","b")
      animal_request = CrvApiClient::Api::Reproduction::ParticipantAnimalRequest.new
      animal_request_list = CrvApiClient::Api::Reproduction::ListParticipantAnimalRequest.new
      requestType = CrvApiClient::Api::Reproduction::ListReproductionLastParityRequestType.new
      service_message = ::CrvApiClient::Api::Reproduction::ServiceMessage
      response = ::CrvApiClient::Api::Reproduction::ListReproductionLastParityResponse.new(nil,service_message)
      driver = configured_client.send(:driver)

      expect(configured_client).to receive(:keeper_and_context_message_detail).with(crv_id, country_code).and_return([keeper,context_message_detail]).once
      expect(configured_client).to receive(:context_message).with(keeper, username: username, password: password, customer: instance_of(CrvApiClient::Api::Reproduction::ParticipantId), provider: instance_of(CrvApiClient::Api::Reproduction::ParticipantId), animal: nil, context_message_detail: context_message_detail,  process_code: process_code, organisation: nil).and_return(context_message).once
      expect(configured_client).to receive(:participant_animal_request).with("TRP","PED").and_return(animal_request).once
      expect(configured_client).to receive(:list_participant_animal_request).with("SHORT",[animal_request]).and_return(animal_request_list).once
      expect(configured_client).to receive(:list_reproduction_last_parity_request_type).with(keeper,animal_request_list).and_return(requestType).once
      expect(driver).to receive(:listReproductionLastParity).with(satisfy { |message|
        expect(message).to be_a(CrvApiClient::Api::Reproduction::ListReproductionLastParityRequest)
        expect(message.requestMessage).to be_a(CrvApiClient::Api::Reproduction::ListReproductionLastParityRequestType)
        expect(message.requestMessage).to be(requestType)
        expect(message.contextMessage).to be(context_message)
      }).and_return(response).once
      expect(configured_client).to receive(:perform_service_message_validation).with(service_message).never

      response = configured_client.list_reproduction_last_parity(crv_id,country_code,process_code: process_code, options: {validate: false})
    end
  end

  context 'list_reproduction_last_parity_request_type' do
    it 'should create a new ListReproductionLastParityRequestType' do
      keeper = ::CrvApiClient::Api::Reproduction::ParticipantId.new
      participant_animal_request = CrvApiClient::Api::Reproduction::ListParticipantAnimalRequest.new
      result = configured_client.send(:list_reproduction_last_parity_request_type, keeper,participant_animal_request)
      expect(result).to be_a(CrvApiClient::Api::Reproduction::ListReproductionLastParityRequestType)
      expect(result.keeper).to be(keeper)
      expect(result.participantAnimalRequest).to be(participant_animal_request)
    end
  end

  context 'list_participant_animal_request' do
    it 'should create a new ListParticipantAnimalRequest' do
      list_type = "SHORT"
      participant_animal_request = CrvApiClient::Api::Reproduction::ParticipantAnimalRequest.new
      result = configured_client.send(:list_participant_animal_request, list_type, participant_animal_request)
      expect(result).to be_a(CrvApiClient::Api::Reproduction::ListParticipantAnimalRequest)
      expect(result.listType).to be(list_type)
      expect(result.participantAnimalRequest).to be(participant_animal_request)
    end
  end

  context 'participant_animal_request' do
    it 'should create a new ListParticipantAnimalRequest' do
      number_type = "TRP"
      number_sub_type= "PED"
      result = configured_client.send(:participant_animal_request, number_type, number_sub_type)
      expect(result).to be_a(CrvApiClient::Api::Reproduction::ParticipantAnimalRequest)
      expect(result.numberType).to be(number_type)
      expect(result.numberSubType).to be(number_sub_type)
    end
  end
  
end